--[[
	Copyright (C) Udorn (Blackhand)

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
--]]

--[[
	Main menu of AuctionMaster displayed in the title bar of the auction house.
--]]

local L = vendor.Locale.GetInstance()
local waterFall = AceLibrary("Waterfall-1.0")

vendor.MainMenu = {}
vendor.MainMenu.prototype = {}
vendor.MainMenu.metatable = {__index = vendor.MainMenu.prototype}

local FRAME_WIDTH=140
local FRAME_HEIGHT=100
local CMD_BUT_WIDTH=120
local CMD_BUT_HEIGHT=12
local CMD_BUT_OFF=2

--[[
	Toggles the position of the AuctionMaster button.
--]]
local function _ToggleButtonPosition(self, initPos)
	self.but:ClearAllPoints()
	if (self.isLeftPos or initPos) then
		self.but:SetPoint("TOPRIGHT", AuctionFrame, "TOPRIGHT", -21, -13);
		self.isLeftPos = false
	else
		self.but:SetPoint("TOPLEFT", AuctionFrame, "TOPLEFT", 67, -13);
		self.isLeftPos = true
	end
end

-- describe the commands contained in the main menu
local CMDS = {
	scan = {
		text = L["Scan"],
		command = function() vendor.Scanner:Scan() end,
		enabled = function() return not vendor.Scanner:IsScanning() end,
		order = 1
	},
	fastScan = {
    	text = L["Fast scan"],
    	command = function() vendor.Scanner:Scan(nil, true) end,
    	enabled = function() return not vendor.Scanner:IsScanning() end,
    	order = 2
	},
	snipe = {
		text = L["Snipe"],
		command = function() vendor.Sniper:OpenSnipeDialog() end,
		order = 3
	},
	conf = {
		text = L["Configuration"],
		command = function() 
			if (waterFall:IsOpen("AuctionMaster")) then 
				waterFall:Close("AuctionMaster"); 
			else
				waterFall:Open("AuctionMaster");
			end
		end,
		order = 4
	},
	togglePos = {
		text = L["Toggle position"],
		command = function(self) _ToggleButtonPosition(self) end,
		order = 5
	},
--	close = {
--		text = "Close",
--		command = function() vendor.Vendor:Debug("Close pressed") end,
--		order = 100
--	},
}

--[[ 
	Sorts according to order value.
--]]
local function _SortCmds(a, b)
	return a.order < b.order
end

--[[
	Adds the commands to the frame.
--]]
local function _AddCmds(self, frame)
	vendor.Vendor:Debug("_AddCmds enter")
	-- sort cmds
	local cmds = {}
	for k,v in pairs(CMDS) do
		table.insert(cmds, v)
	end
	table.sort(cmds, _SortCmds)
	-- add buttons
	local off = 1
	for k,v in pairs(cmds) do
		vendor.Vendor:Debug("add cmd: "..v.text)
		local butName = "VendorMainMenuCmd"..off
		local but = CreateFrame("Button", butName, frame, "UIDropDownMenuButtonTemplate")
		but.ctrl = self; 
		but:SetWidth(CMD_BUT_WIDTH);
		but:SetHeight(CMD_BUT_HEIGHT);
		but:SetPoint("TOPLEFT", 10, -((off - 1) * (CMD_BUT_HEIGHT + CMD_BUT_OFF) + 5));
		but:SetText(v.text);
		but.ctrl = self
		but.cmd = v
		but.prevOnLeave = but:GetScript("OnLeave")
		but.prevOnEnter = but:GetScript("OnEnter")
		but:SetScript("OnClick", function(but) but.ctrl.frame:Hide(); but.cmd.command(but.ctrl) end)
		but:SetScript("OnLeave", function(but) 
			but.ctrl.frame.butEntered = false
			if (but.prevOnLeave) then but.prevOnLeave(but) end 
		end)
    	but:SetScript("OnEnter", function(but) 
    		but.ctrl.frame.butEntered = true
    		if (but.prevOnEnter) then but.prevOnEnter(but) end 
    	end)
    	--but:SetScript("OnShow", function(but) but.ctrl.frame.butEntered = true; vendor.Vendor:Debug("but show") end)
		if (v.enabled) then
			but:SetScript("OnUpdate", function(but)
				if (but.cmd.enabled()) then
					but:Enable()
				else
					but:Disable()
				end 
			end)
		end
		local checkImage = getglobal(butName.."Check");
		checkImage:Hide()
		off = off + 1
	end
	frame:SetHeight(off * (CMD_BUT_HEIGHT + CMD_BUT_OFF) - CMD_BUT_OFF)
	vendor.Vendor:Debug("_AddCmds exit")
end

--[[
	Initializes the frame.
--]]
local function _InitFrame(self)
	vendor.Vendor:Debug("_InitFrame enter")
    local frameName = "VendorMainMenuFrame";
    local frame = CreateFrame("Frame", frameName, UIParent, "VendorDialogTemplate");
    frame.ctrl = self;
    self.frame = frame;
    frame:SetWidth(FRAME_WIDTH);
    frame:SetHeight(FRAME_HEIGHT);
    frame:SetPoint("TOPLEFT", self.but, "TOPLEFT", 0, 0);
    frame:SetFrameStrata("DIALOG");
    frame:SetToplevel(true);
    frame:EnableMouse(true)
    frame:SetScript("OnLeave", function(frame) frame.entered = false end)
    frame:SetScript("OnEnter", function(frame) frame.entered = true end)
    frame:SetScript("OnShow", function(frame) frame.entered = true end)
    frame:SetScript("OnUpdate", function(frame)
    	if (not frame.entered and not frame.butEntered) then
    		frame:Hide()
    	end
    end)
    _AddCmds(self, frame)
    frame:Hide()
    vendor.Vendor:Debug("_InitFrame exit")
end

--[[
	Create the main menu dropdown.
--]]
local function _CreateButton(self)
	local but = CreateFrame("Button", nil, AuctionFrame, "UIPanelButtonTemplate")
	but:SetText(L["AuctionMaster"])
	but:SetWidth(CMD_BUT_WIDTH)
	but:SetHeight(22)
	self.but = but;
	_ToggleButtonPosition(self, true)
	but.ctrl = self
	but:SetScript("OnClick", function(but) but.ctrl.frame:Show() end)
end

--[[
	Creates a new instance.
--]]
function vendor.MainMenu:new(...)
	local instance = setmetatable({}, self.metatable)
	instance.nesting = 0
	_CreateButton(instance)
	_InitFrame(instance)
	return instance
end

--[[
	Shows the main menu button
--]]
function vendor.MainMenu.prototype:Show()
	self.but:Show()
end

--[[
	Hides the main menu button
--]]
function vendor.MainMenu.prototype:Hide()
	self.but:Hide()
end
